/*eslint-env node*/
var Tipit = function () {
  this.questions = {
    extraverted: {'name': '', 'description': 'extraverted, enthusiastic', 'value': 0},
    critical: {'name': 'critical', 'description': 'critical, quarrelsome', 'value': 0},
    dependable: {'name': 'dependable', 'description': 'dependable, self-disciplined', 'value': 0},
    anxious: {'name': 'anxious', 'description': 'anxious, easily upset', 'value': 0},
    openess: {'name': 'openess', 'description': 'open to new experiences, complex', 'value': 0},
    reserved: {'name': 'reserved', 'description': 'reserved, quiet', 'value': 0},
    sympathetic: {'name': 'sympathetic', 'description': 'sympathetic, warm', 'value': 0},
    disorganized: {'name': 'disorganized', 'description': 'disorganized, careless', 'value': 0},
    calm: {'name': 'calm', 'description': 'calm, emotionally stable', 'value': 0},
    conventional: {'name': 'conventional', 'description': 'conventional, uncreative', 'value': 0}
  };
  this.recodeQuestions = ['critical','anxious','reserved','disorganized','conventional'];
  this.values = {1:7, 2:6, 3:5, 4:4, 5:3, 6:2, 7:1};
  this.results = {};

  return this;
};

/**
 * makeQuestionResult adds together two question values by name.
 * @param  string questionName1
 * @param  string questionName2
 * @return void
 */
Tipit.prototype.makeQuestionResult = function (questionName1, questionName2) {
  return (this.questions[questionName1].value + this.questions[questionName2].value);
};

/**
* checkValues method to ensure all questions have a value assigned
* @return obj  returns current Tipit instance
*/
Tipit.prototype.checkValues = function () {
  var unsetAr = [];

  var question = {};
  for (question in this.questions) {
    if (this.questions.hasOwnProperty(question)) {
      question = this.questions[question];
      if (question.value === 0) unsetAr.push(question.name);
    }
  }

  if (unsetAr.length > 0) throw Error("Question indices " + unsetAr.join(', ') + " are unset.");
  return this;
};

/**
* recode method to perform value recoding, effectivly inverting some values
* based on the this.recodes array in prep for calculation
* following the alogorithm from the taken from the ten item inventory test
* @return obj  returns current Tipit instance
*/
Tipit.prototype.recode = function () {
  var question = {};
  for (question in this.questions) {
    if (this.questions.hasOwnProperty(question)) {
      question.value = this.values[question.value];
    }
  }
  return this;
};

/**
* Method for setting value of question by name
* value must be a number between 1 and 7.
* 1 meaning strongly disagree, 7 meaning strongly agree.
* @param string    questionName    string index of this.questions
* @param number    value           number to be assigned to question
*/
Tipit.prototype.set = function (questionName, value) {
  // check question index/name exists in questions
  if (! this.questions.hasOwnProperty(questionName)) {
    throw new Error('Unknown index ['+questionName+'] Possible question indexes are; extraverted, critical, dependable, anxious, openess, reserved, sympathetic, disorganized, calm, conventional');
  }
  // check value submitted is valid
  if ((typeof value !== 'number') || value < 0 || value > 7) {
    throw new Error('Values range from 1 to 7 and must be a number.');
  }

  this.questions[questionName].value = value;
  return this;
};

/**
 * Main calculation method, used after all questions have been 'set()''
 * @return obj  returns current Tipit instance
 */
Tipit.prototype.calculate = function() {
    this.checkValues()
        .recode()
        .calculateBigFiveTraits()
        .estimateBriggsType()
        .estimateKeirseyTemprement();

    return this.results;
  };

  /**
   * calculateBigFiveTraits, does what the name implies
   * @return obj  returns current Tipit instance
   */
Tipit.prototype.calculateBigFiveTraits = function () {
    this.results.big_five_traits = {
      'extraversion'              : this.makeQuestionResult('extraverted', 'reserved'),
      'open_to_experiences'   : this.makeQuestionResult('openess', 'conventional'),
      'agreeableness'             : this.makeQuestionResult('critical', 'sympathetic'),
      'conscientiousness'         : this.makeQuestionResult('disorganized', 'dependable'),
      'emotional_stability'  : this.makeQuestionResult('anxious', 'calm')
    };

    return this;
  };

  /**
   * estimateBriggsType, does what the name implies
   * @return obj  returns current Tipit instance
   */
  Tipit.prototype.estimateBriggsType = function () {
    var briggsType = [
      this.results.big_five_traits.extraversion < 3.5 ? 'i' : 'e',
      this.results.big_five_traits.openess_to_experiences < 3.5 ? 's' : 'n',
      this.results.big_five_traits.agreeableness < 3.5 ? 't' : 'f',
      this.results.big_five_traits.conscientiousness < 3.5 ? 'p' : 'j',
    ];

    this.results.estimated_briggs_type = briggsType.join('');

    return this;
  };

  /**
   * estimateKeirseyTemprement, does what the name implies
   * @return obj  returns current Tipit instance
   */
  Tipit.prototype.estimateKeirseyTemprement = function () {
    var keirsey = {
        ISTP: {code: 'ISTP', temprement: 'artisan', strength: 'tactical', intelligence_role: 'operator', operation: 'expediting', role: 'crafter'},
        ISFP: {code: 'ISFP', temprement: 'artisan', strength: 'tactical', intelligence_role: 'entertainer', operation: 'improvising', role: 'composer'},
        ESTP: {code: 'ESTP', temprement: 'artisan', strength: 'tactical', intelligence_role: 'operator', operation: 'expediting', role: 'promoter'},
        ESFP: {code: 'ESFP', temprement: 'artisan', strength: 'tactical', intelligence_role: 'entertainer', operation: 'improvising', role: 'performer'},
        ISTJ: {code: 'ISTJ', temprement: 'guardian', strength: 'logistical', intelligence_role: 'administrator', operation: 'regulating', role: 'inspector'},
        ISFJ: {code: 'ISFJ', temprement: 'guardian', strength: 'logistical', intelligence_role: 'conservator', operation: 'supporting', role: 'protector'},
        ESTJ: {code: 'ESTJ', temprement: 'guardian', strength: 'logistical', intelligence_role: 'administrator', operation: 'regulating', role: 'supervisor'},
        ESFJ: {code: 'ESFJ', temprement: 'guardian', strength: 'logistical', intelligence_role: 'conservator', operation: 'supporting', role: 'provider'},
        INFJ: {code: 'INFJ', temprement: 'idealist', strength: 'diplomatic', intelligence_role: 'mentor', operation: 'developing', role: 'counsellor'},
        INFP: {code: 'INFP', temprement: 'idealist', strength: 'diplomatic', intelligence_role: 'advocate', operation: 'mediating', role: 'healer'},
        ENFP: {code: 'ENFP', temprement: 'idealist', strength: 'diplomatic', intelligence_role: 'advocate', operation: 'mediating', role: 'champion'},
        ENFJ: {code: 'ENFJ', temprement: 'idealist', strength: 'diplomatic', intelligence_role: 'mentor', operation: 'developing', role: 'teacher'},
        INTJ: {code: 'INTJ', temprement: 'rational', strength: 'strategic', intelligence_role: 'coordinator', operation: 'arranging', role: 'mastermind'},
        INTP: {code: 'INTP', temprement: 'rational', strength: 'strategic', intelligence_role: 'engineer', operation: 'constructing', role: 'architect'},
        ENTP: {code: 'ENTP', temprement: 'rational', strength: 'strategic', intelligence_role: 'engineer', operation: 'constructing', role: 'inventor'},
        ENTJ: {code: 'ENTJ', temprement: 'rational', strength: 'strategic', intelligence_role: 'coordinator', operation: 'arranging', role: 'fieldmarshal'},
    },
    briggs = this.results.estimated_briggs_type.toUpperCase();

    this.results.estimated_keirsey_results = keirsey[briggs];

  };

/**
 * constructor
 */
//Tipit.prototype.constructor = Tipit;


/** Conditional export if module.exports is supported **/
if ( typeof module === "object" && module && typeof module.exports === "object" ) {
  module.exports = Tipit;
}
